import React, { useState, useEffect, PureComponent } from 'react';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var createState = require("jstates");
function subscribe(Compt, statesToSubscribeTo, mapStatesToProps) {
  if (mapStatesToProps === void 0) {
    mapStatesToProps = function mapStatesToProps(states) {
      return {
        states: states
      };
    };
  }

  if (!Compt) {
    throw new Error("subscribe was called without a component. subscribe(Component, [statesToSubscribeTo], mapStatesToProps)");
  }

  if (!statesToSubscribeTo) {
    throw new Error("subscribe was called without states to subscribe to. subscribe(Component, [statesToSubscribeTo], mapStatesToProps)");
  }

  statesToSubscribeTo = Array.isArray(statesToSubscribeTo) ? statesToSubscribeTo : [statesToSubscribeTo];
  return (
    /*#__PURE__*/
    function (_PureComponent) {
      _inheritsLoose(Subscribe, _PureComponent);

      function Subscribe(props) {
        var _this;

        _this = _PureComponent.call(this, props) || this;
        _this.mounted = true;
        _this.onUpdate = _this.onUpdate.bind(_assertThisInitialized(_this));
        statesToSubscribeTo.forEach(function (state) {
          state.subscribe(_this.onUpdate);
        });
        _this.state = _this.generateState();
        return _this;
      }

      var _proto = Subscribe.prototype;

      _proto.generateState = function generateState() {
        return mapStatesToProps.apply(void 0, statesToSubscribeTo.map(function (i) {
          return i.state;
        }));
      };

      _proto.componentWillUnmount = function componentWillUnmount() {
        var _this2 = this;

        this.mounted = false;
        statesToSubscribeTo.forEach(function (state) {
          state.unsubscribe(_this2.onUpdate);
        });
      };

      _proto.onUpdate = function onUpdate() {
        var _this3 = this;

        return new Promise(function (resolve) {
          if (_this3.mounted) {
            _this3.setState(_this3.generateState(), resolve);
          } else {
            resolve();
          }
        });
      };

      _proto.render = function render() {
        return React.createElement(Compt, _extends({}, this.props, this.state));
      };

      return Subscribe;
    }(PureComponent)
  );
}
var useSubscribe = function useSubscribe(stateInstance) {
  var _useState = useState(stateInstance && stateInstance.state),
      subscribeState = _useState[0],
      setSubscribeState = _useState[1];

  useEffect(function () {
    if (!stateInstance) {
      throw new Error("useSubscribe was called without a state. It should be called like this: useSubscribe(stateInstance);");
    }

    var updateState = function updateState() {
      setSubscribeState(stateInstance.state);
    };

    stateInstance.subscribe(updateState);
    return function () {
      stateInstance.unsubscribe(updateState);
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  return subscribeState;
};

export { createState, subscribe, useSubscribe };
